require "social_avatar_proxy/configuration/file_cache"
require "social_avatar_proxy/configuration/http_cache"
require "social_avatar_proxy/configuration/memcache"
require "social_avatar_proxy/configuration/caches"
require "social_avatar_proxy/avatar_file"

module SocialAvatarProxy
  class Configuration
    def initialize
      @memcache = Memcache.new
      @file_cache = FileCache.new
      @http_cache = HttpCache.new
    end
    
    # updates the configuration
    def configure(&block)
      instance_eval(&block)
    end

    # configures or retrieves the file cache instance
    def file_cache(&block)
      if block_given?
        @file_cache.configure(&block)
      else
        @file_cache
      end
    end

    # configures or retrieves the http cache instance
    def http_cache(&block)
      if block_given?
        @http_cache.configure(&block)
      else
        @http_cache
      end
    end
    
    # configures or retrieves the memcache cache instance
    def memcache(&block)
      if block_given?
        @memcache.configure(&block)
      else
        @memcache
      end
    end
    
    # serves the file via a header
    def x_send_file
      serve_via(:x_send_file)
    end
    
    # serves the file via an x_accel_redirect
    def x_accel_redirect
      serve_via(:x_accel_redirect)
    end
    
    # returns how the app should serve the file
    # defaults to streaming the file data
    def delivery_method
      @serve_via ||= :stream
    end
    
    # returns a Caches object containing the configured caches
    def caches
      Caches.new.tap do |set|
        set.push(@memcache) if @memcache.enabled?
        set.push(@file_cache) if @file_cache.enabled?
      end
    end
    
    def default_image(value = nil)
      if value
        unless File.exist?(value) && File.readable?(value)
          raise ArgumentError, "#{value} does not exist, or is unreadable"
        end
        @default_image = value
      end
      @default_image && AvatarFile.new(@default_image).tap do |file|
        file.content_type(default_image_content_type)
      end
    end
    
    def default_image_content_type(value = nil)
      @default_image_content_type = value if value
      [
        @default_image_content_type,
        auto_detect_default_image_content_type,
        "application/octet-stream"
      ].compact.first
    end
    
    private
    def serve_via(value)
      @serve_via = value
    end
    
    def auto_detect_default_image_content_type
      case File.extname(@default_image)
      when ".png"
        "image/png"
      when ".gif"
        "image/gif"
      when ".svg"
        "application/svg+xml"
      when /\.jpe?g/
        "image/jpeg"
      else
        "application/octet-stream"
      end if @default_image
    end
  end
end